module smooth_modDB

  use sep

  implicit none

contains

!!--------------------------------------------------------------------------
!! Subroutine that do box smoothing (used for denominator, code from BEI)

  subroutine boxconv( nb, nx, xx, yy)
! inputs:       nx,  xx(i), i=1,nx      the data
!               nb                      the box length
! output:       yy(i),i=1,nx+nb-1       smoothed data

    integer                 :: nx, ny, nb, i
    real*8, dimension(:)      :: xx, yy  ! xx(nx), yy(1)
    real*8, dimension(:), allocatable :: bb
    
    allocate(bb(nx+nb));

    if( nb < 1 .or. nb > nx)  call erexit('boxconv');

    ny = nx+nb-1

    do i= 1, ny
       bb(i) = 0.
       bb(1) = xx(1)
    end do

    do i= 2, nx
       bb(i) = bb(i-1) + xx(i)         ! make B(Z) = X(Z)/(1-Z)
    end do

    do i= nx+1, ny
       bb(i) = bb(i-1)
    end do

    do i= 1, nb
       yy(i) = bb(i)
    enddo

    do i= nb+1, ny
       yy(i) = bb(i) - bb(i-nb)        ! make Y(Z) = B(Z)*(1-Z**nb)
    end do

    do i= 1, ny
       yy(i) = yy(i) / nb
    enddo

    deallocate(bb);
  
  end subroutine boxconv


!!--------------------------------------------------------------------------
!! Subroutine that do 1D triangle smoothing  (code from BEI)

! Convolve with triangle
  
  subroutine triangle( nr, n12, uu, vv)
! input:        nr      rectangle width (points) (Triangle base twice as wide.)
! input:        uu(i2),i2=1,n12      is a vector of data.
! output:       vv(i2),i2=1,n12      may be on top of uu
 
    integer                    :: nr,n12, i,np,nq
    real*8, dimension(:)       :: uu, vv            ! uu( m1, n12),  vv( m1, n12)        
    real*8, dimension(:), allocatable ::  pp, qq, tt

    allocate(pp(n12+nr-1), qq(n12+nr+nr-2), tt(n12) );
    
    do i=1,n12
       qq(i) = uu(i) 
    end do
    
    if( n12 == 1 ) then
       do i= 1, n12
          tt(i) = qq(i)
       end do
    else 

       call boxconv( nr, n12, qq, pp);         np = nr+n12-1
       call boxconv( nr, np , pp, qq);         nq = nr+np-1

       do i= 1, n12
          tt(i) = qq(i+nr-1)
       end do

       do i= 1, nr-1                                   ! fold back near end
          tt(i) = tt(i) + qq(nr-i)
       end do

       do i= 1, nr-1                                   ! fold back far end
          tt(n12-i+1) = tt(n12-i+1) + qq(n12+(nr-1)+i)
       end do

    end if

    do i=1,n12 
       vv(i) = tt(i) 
    enddo

    deallocate(pp, qq, tt);

  end subroutine triangle


!!--------------------------------------------------------------------------
! smooth by convolving with triangle in two dimensions.
! input: rec1 = rectangle width (triangle width is twice as wide) for axis 1
!        rec2 (for axis 2)
!        n1 = dimension of axis 1;    n2= dimension of axis 2
!        uu(n1,n2) input array to be smoothed
! Output: vv(n1,n2)  

  subroutine triangle2( rect1, rect2, n1, n2, uu, vv)

    integer                       :: i1,i2        
    integer                       :: rect1, rect2, n1, n2
    real*8, dimension(:,:)          :: uu, vv !uu(n1,n2), vv(n1,n2)
    real*8, dimension(:,:), allocatable :: ss
  
    allocate(ss(n1,n2));


    do i1= 1, n1
       call triangle( rect2, n2, uu(i1,:), ss(i1,:))
    end do


    do i2= 1, n2
       call triangle( rect1, n1, ss(:,i2), vv(:,i2))
    end do


    deallocate(ss);
    
    return; 

  end subroutine triangle2

!-----------------------------------------------------------





end module
